@@ -38,7 +38,7 @@ class DirProc(FsTaskProcessor):
     def getrec(self, path, args, depth, symlinks=[]):
         # print("! getrec:",path)
         out = []
-        
+
         rec=dict()
         rec['path']=os.path.realpath(path)
         rec['basename']=os.path.basename(path)
@@ -51,19 +51,19 @@ class DirProc(FsTaskProcessor):
         rec['atime']=int(s.st_atime)
         rec['ctime']=int(s.st_ctime)
         rec['mtime']=int(s.st_mtime)
-        
+
         now = time.time()
         rec['aage']=int(now-s.st_atime)
         rec['mage']=int(now-s.st_mtime)
         rec['cage']=int(now-s.st_ctime)
-        
 
-        
+
+
         if stat.S_ISDIR(s.st_mode):
-            rec['type']="DIR"          
-            # should we go deeper?        
-            
-            if not args['maxdepth'] or int(args['maxdepth'])>depth:                
+            rec['type']="DIR"
+            # should we go deeper?
+
+            if not args['maxdepth'] or int(args['maxdepth'])>depth:
                 for basename in os.listdir(path):
                     fullname = os.path.join(path,basename)
                     subout = self.getrec(fullname,args,depth+1, symlinks)
@@ -82,48 +82,48 @@ class DirProc(FsTaskProcessor):
             if args['sha256']:
                rec['sha256'] = hashfile(open(path,'rb'), hashlib.sha256())
 
-            
+
         elif stat.S_ISFIFO(s.st_mode):
             rec['type']="FIFO"
         elif stat.S_ISLNK(s.st_mode):
-            if args['symlink']=='0':               
+            if args['symlink']=='0':
                 rec['type']="LNK"
             else:
-            
+
                 target = os.path.realpath(path)
-                
+
                 if target in symlinks:
                     return []
-                
+
                 symlinks.append(target)
-                return self.getrec(target ,args,depth+1, symlinks)                
+                return self.getrec(target ,args,depth+1, symlinks)
                 return []
-                
+
         elif stat.S_ISSOCK(s.st_mode):
             rec['type']="SOCK"
-                                       
+
         if depth>=int(args['mindepth']):
-            out.append(rec)                    
+            out.append(rec)
         return out
 
-    
+
     def run(self,ts,data,args):
-        self.ts = ts                   
+        self.ts = ts
         if not isinstance(data,list):
             data=[]
         if args['path']:
             try:
-                data.extend(self.getrec(args['path'],args,0))        
+                data.extend(self.getrec(args['path'],args,0))
             except (IOError, OSError, PermissionError, FileNotFoundError) as e:
-                self.ts.oc.log.debug("exception:"+str(e))
+                self.ts.oc.log.debug("exception: {}".format(e))
                 ts.stop()
                 return None
-                    
+
         else:
             self.ts.oc.log.error('DIR requires path, e.g. DIR path=/var/log or DIR path=/var/log/mail.log')
         return data
 
-DirProc('DIR',ts.TaskSeq)        
+DirProc('DIR',ts.TaskSeq)
 
 class DFProc(FsTaskProcessor):
     help='free disk space'
@@ -134,7 +134,7 @@ class DFProc(FsTaskProcessor):
         g=1024*1024*1024 # 1GB
 
         d={}
-            
+
         d['total']=u.total
         d['used']=u.used
         d['free']=u.free
@@ -143,46 +143,46 @@ class DFProc(FsTaskProcessor):
         d['usedg']=int(u.used/g)
         d['freeg']=int(u.free/g)
 
-        
+
         d['percent']=u.percent
         return d
-    
+
     @staticmethod
     def inodes(d):
         st = os.statvfs(d['path'])
-        
+
         d['inodes_total'] = st.f_files
         d['inodes_free'] = st.f_ffree
-        if st.f_files:            
-            d['inodes_percent'] = ((st.f_files - st.f_ffree) * 100) / st.f_files                    
+        if st.f_files:
+            d['inodes_percent'] = ((st.f_files - st.f_ffree) * 100) / st.f_files
         else:
             d['inodes_percent'] = 0
-    
+
     def run(self,ts,data,args):
         self.ts = ts
 
         if args['path']:
-            u = psutil.disk_usage(args['path'])            
+            u = psutil.disk_usage(args['path'])
             d = DFProc.usage2g(u)
             d['path']=args['path']
             DFProc.inodes(d)
             return d
         else:
-            dflist=[]            
+            dflist=[]
             points=[]
             # generate mount points which we willl watch
             for p in psutil.disk_partitions():
-                
+
                 if p.device.startswith('/dev/loop'):
-                    continue 
+                    continue
                 points.append(p.mountpoint)
-            
+
             #
             # BUGFIX: openvz root simfs is not listed in points
             #
             if not points:
                 points.append('/')
-            
+
             for point in points:
                 try:
                     u = psutil.disk_usage(point)
@@ -190,20 +190,42 @@ class DFProc(FsTaskProcessor):
                     d['path']=point
                     DFProc.inodes(d)
                     dflist.append(d)
-                    
+
                 except OSError as e:
-                    self.ts.oc.log.warn('Have no access to mount {}'.format(point))                    
+                    self.ts.oc.log.warn('Have no access to mount {}'.format(point))
             return dflist
 
-DFProc('DF',ts.TaskSeq)        
-            
+DFProc('DF', ts.TaskSeq)
+
+class DUProc(FsTaskProcessor):
+    help='disk usage'
+    defargs={
+        'path': '',
+        #'depth': '0',
+        #'sep': '-'
+        }
+
+    def run(self,ts,data,args):
+        total = 0
+        # print "DU for path", args['path']
+        for root, dirs, files in os.walk(str(args['path']), topdown=False):
+            #print "root: {} dirs: {}, files: {}".format(root, dirs, files)
+            for f in files:
+                filepath = os.path.join(root,f)
+                if os.path.isfile(filepath) and not os.path.islink(filepath):
+                    # print "{} {}".format(os.path.getsize(filepath), filepath)
+                    total += os.path.getsize(filepath)
+        return total
+
+DUProc('DU', ts.TaskSeq)
+
 
 class FileProc(FsTaskProcessor):
     help='Read file'
 
     store_argline='path'
     parse_args=False
-    
+
     defargs={'path': ''}
 
     tpconf={
@@ -211,29 +233,29 @@ class FileProc(FsTaskProcessor):
     }
 
     def run(self,ts,data,args):
-    
+
         path = args['path']
 
         # check if path in whitelist
-    
+
         if ts.oc.tpconf_enabled:
             if not '*' in self.tpconf['read']:
                 if not path in self.tpconf['read']:
                     # error! cannot read this file!
-                    tpconfline = 'Maybe try --tpconf {}:read={}'.format(self.code, path)             
+                    tpconfline = 'Maybe try --tpconf {}:read={}'.format(self.code, path)
                     ts.stop("File {} is not allowed to read. Whitelist read files: {}. Maybe try: {} ?".format(path,self.tpconf['read'], tpconfline))
                     return None
 
-        try:                    
+        try:
             fh = open(path)
         except IOError as e:
-            ts.oc.log.error('Cannot read file \'{}\': {}'.format(path, str(e)))
+            ts.oc.log.error('Cannot read file \'{}\': {}'.format(path, e))
             ts.stop()
             return None
-            
-        return fh        
-        
-FileProc('FILE',ts.TaskSeq)        
+
+        return fh
+
+FileProc('FILE',ts.TaskSeq)
 
 
 class JFileProc(ts.TaskAlias):
@@ -242,12 +264,12 @@ class JFileProc(ts.TaskAlias):
 
     store_argline='path'
     parse_args=False
-    
+
     defargs={'path': ''}
 
     alias = ['FILE','MULTILINE','FROMJSON']
 
-JFileProc('JFILE',ts.TaskSeq)    
+JFileProc('JFILE',ts.TaskSeq)
 
 
 
@@ -257,10 +279,10 @@ class SeqFileProc(ts.TaskAlias):
 
     store_argline='path'
     parse_args=False
-    
+
     defargs={'path': ''}
 
     # alias = ['FILE','LIST','MKSEQ']
     alias = ['FILE','LIST','MKSEQ']
 
-SeqFileProc('SEQFILE',ts.TaskSeq)    
+SeqFileProc('SEQFILE',ts.TaskSeq)
