@@ -55,10 +55,10 @@ class pidfile:
     def __exit__(self, etype, evalue, etb):
         if etype is not None and etype != SystemExit:
             self.log("{} etype: {}".format(os.getpid(),str(etype)))
-#            self.log("TB:"+etb.format_exc())
-            traceback.print_tb(etb)
+            self.log("exception:"+etb.format_exc())
+            self.log("traceback:"+traceback.extract_tb(etb))
         
-        self.log("{} pid exit".format(os.getpid()))   
+        self.log("{} pid exit".format(os.getpid()))
         if self.lock.acquired:
             self.lock.release()
             os.unlink(self.lockpath)
