@@ -34,7 +34,7 @@ class TaskProcessor(object):
     _help='help stub'
     help_suffix = None
     code = '<No Code>'
-    
+
     store_argline=None # if set, whole argline stored in this key in args{} dict
     store_cmdline=None # same, but with method
     parse_args=True # set False to not parse args. (e.g. if method has just one required arg)
@@ -52,8 +52,8 @@ class TaskProcessor(object):
     def run(self,ts,data,args):
         print('YOU MUST OVERRIDE RUN')
         return 123456
-    
-    @property   
+
+    @property
     def help(self):
         return 'ZZZ' + self._help
 
@@ -67,53 +67,53 @@ class TaskProcessor(object):
 
 
     def fullhelp(self):
-        
+
         msg = ""
         msg += self.code+'\t'+self.help+'\n'
-        
+
         if self.tpconf:
             msg += '\tconfiguration:\n'
             for cname in sorted(self.tpconf):
                 msg += '\t{} (--tpconf {}:{}=...): {}\n'.format(cname,self.code,cname,self.tpconf[cname])
             msg+='\n'
-        
-        if self.parse_args and self.defargs:                    
+
+        if self.parse_args and self.defargs:
             msg += '\tArguments:\n';
             for aname in sorted(self.defargs):
                 msg += '\t'+aname+'='+repr(self.defargs[aname])+'\n'
         if self.store_argline:
             msg+='\tparameter ({}), default: "{}"\n'.format(self.store_argline,self.defargs[self.store_argline])
-            
+
         if self.help_suffix:
             msg+=self.help_suffix
-            
+
         return msg
-        
+
     def tpconfset(self,k,v):
         if k in self.tpconf and isinstance(self.tpconf[k],six.string_types):
             self.tpconf[k]=v
         elif k in self.tpconf and isinstance(self.tpconf[k],list):
             self.tpconf[k].append(v)
         else:
-            self.tsclass.oc.log.error('Task Processor \'{}\' has no configuraton key \'{}\''.format(self.code,k))            
-          
+            self.tsclass.oc.log.error('Task Processor \'{}\' has no configuraton key \'{}\''.format(self.code,k))
+
 
     def __init__(self, code, tsclass):
         self.code = code
         tsclass.addtproc(code,self)
-   
+
     def parseargs(self, args):
         ad = dict(self.defargs) # copy default dictionary
-        
+
         # set method
         ad['command']=self.code
-        
+
         if self.parse_args:
-            va = shlex.split(args)[1:]        
+            va = shlex.split(args)[1:]
             for v in va:
-                kv = v.split('=',1)                                
-        
-                if kv[0] in ad:                    
+                kv = v.split('=',1)
+
+                if kv[0] in ad:
                     if len(kv)==2:
                         ad[kv[0]]=kv[1]
                     else:
@@ -135,20 +135,21 @@ class TaskProcessor(object):
         if args[name]=='0':
             return False
         return True
-    
-    
-    
+
+
+
 
 #
 # route just array of text, like [ "KEYVAL", "JDUMP"]
 #
 #
 #
-    
+
 class TaskSeq():
     oc = None # okerr client object
     iname = None # indicator
     method = None # method of indicator
+    policy = None # policy
     details = None # details
     except_data = None # used for exception
 
@@ -158,12 +159,12 @@ class TaskSeq():
     # keypath is dynamical part, e.g. df
     # full keypath is combination: servers:srv1:df
     #
-    
+
     keypath = None
     origkeypath = None
 
     tags=[]
-    
+
     v = {}
 
     #methods={'OK': self.ok,'ERR': self.err, 'STR': self.string }
@@ -174,11 +175,11 @@ class TaskSeq():
     _stop = False
 
     steps = None # if not none, stop after this N of steps
-    
-    dump = None # method of debug dump 
+
+    dump = None # method of debug dump
     dumpname = None
     lastcmd = None
-    
+
     def stop(self, details=None):
         if details:
             self.oc.error(details)
@@ -187,7 +188,10 @@ class TaskSeq():
 
     # taskseq.setsteps
     def setsteps(self,steps):
-        self.steps=steps
+        if steps.upper() == 'ALL':
+            self.trace = True
+        else:
+            self.steps=int(steps)
 
     # taskseq.setdump
     def setdump(self,dump):
@@ -198,15 +202,15 @@ class TaskSeq():
         self.dumpname=dumpname
 
 
-    
+
     #
     # TaskSeq.mkargs
     #
     # convert element ('OK' or 'ANYCODE arg1..', or dict)
     # to route element
-    # 
+    #
     def mkargs(self,s):
-              
+
         if isinstance(s,six.string_types):
             try:
                 m = shlex.split(s)[0]
@@ -214,51 +218,51 @@ class TaskSeq():
                 raise ValueError('Cannot parse command: {}'.format(s))
                 return None
             #self.oc.log.debug('parsing command {}'.format(s))
-            
-            
+
+
             if not m in self.tp:
                 raise OkerrBadMethod('No method '+m)
-                                   
+
             # MK ARGS
             args = self.tp[m].parseargs(s)
-                                    
+
             #self.oc.log.debug('args: {}'.format(args))
             return args
-            
+
         elif isinstance(s,dict):
             if not 'command' in s:
                 raise ValueError
             if not s['command'] in self.tp:
-                raise OkerrBadMethod('no method {}'.format(s['command']))        
-                
+                raise OkerrBadMethod('no method {}'.format(s['command']))
+
             # fill defargs
             code = s['command']
             for k in self.tp[code].defargs:
                 if not k in s:
                     s[k]=self.tp[code].defargs[k]
-            
+
             if self.tp[code].store_argline and 'argline' in s:
                 s[self.tp[code].store_argline] = s['argline']
-            
-            return s    
+
+            return s
 
 
     #
-    # TaskSeq.setseq - set sequence   
+    # TaskSeq.setseq - set sequence
     # seq is always list.
     # element is either string like 'OK' or 'ANYCODE arg1=val1 arg2=val2'
     # or element is dict with fields 'cmd' and argnames
     #
 
     def setseq(self,seq):
-        
+
         route = list()
-        
-        for s in seq:            
-            args = self.mkargs(s)            
+
+        for s in seq:
+            args = self.mkargs(s)
             route.append(args)
-    
-    
+
+
         self.fullroute=list(route) # copy it
         self.route = list(route) # copy it
         self.checkroute()
@@ -269,14 +273,14 @@ class TaskSeq():
     #
 
     def __init__(self, iname=None, route=None,method=None):
-        
+
         if iname is None:
             self.iname = self.oc.defname
         else:
             self.iname = iname
-                        
+
         self.dump = ['SEQDUMP']
-        
+
         self.initvars()
 
         if route is not None:
@@ -284,41 +288,45 @@ class TaskSeq():
 
         self.method=method
         self.tags=list()
-        
+
         self.keypath = list()
         self.origkeypath = list()
-        
+
+        self.trace = False
+
     def initvars(self):
         pass
-  
+
 
     #
     # TaskSeq.fork
-    # 
+    #
 
     def fork(self,iname=None,route=None,method=None):
-        ts = TaskSeq(iname,route,method)        
-        
+        ts = TaskSeq(iname,route,method)
+
         if iname is None:
             iname = self.iname
         if method is None:
-            method=self.method        
-            
+            method=self.method
+
 
         ts.iname = iname
-        ts.oc = self.oc        
+        ts.oc = self.oc
         ts.method = method
+        ts.policy = self.policy
         ts.details = self.details
         ts.except_data = self.except_data
         ts.parent = self
         ts.tags = list(self.tags)
         ts.keypath = self.keypath
         ts.origkeypath = self.origkeypath
-        
+
         ts.dump = self.dump
         ts.dumpname = self.dumpname
         ts.steps = self.steps
-        
+        ts.trace = self.trace
+
         return ts
 
 
@@ -329,12 +337,13 @@ class TaskSeq():
 
         return True
 
+    # TaskSeq.setvar
     def setvar(self,name,value):
         if(type(value) is str):
             value = unicode(value,"utf8")
         self.v[name]=value
 
-
+    # TaskSeq.getvar
     def getvar(self,name):
         if name in self.v:
             return self.v[name]
@@ -345,11 +354,14 @@ class TaskSeq():
        v = self.v.copy()
        v['_name']=self.iname
        return v
-        
+
+    def reset_vars(self):
+        self.v = dict() 
+
     def settag(self,tagname):
         if re.match('[a-zA-Z0-9_]*',tagname):
             self.tags.append(tagname)
-        
+
     @classmethod
     def addtproc(cls, code, tproc):
         # check validity
@@ -359,9 +371,9 @@ class TaskSeq():
         else:
             # method is not child of TaskProcessor
             raise BadMethodException()
-            
-            
-    # taskseq.help            
+
+
+    # taskseq.help
     @classmethod
     def help(cls):
 
@@ -372,12 +384,12 @@ class TaskSeq():
             msg+='\n'
             return msg
 
-    
+
         chapters={}
-        
+
         for code in cls.tp:
             c = cls.tp[code]
-            
+
             if not c.chapter in chapters:
                 chapters[c.chapter] = [c]
             else:
@@ -388,7 +400,7 @@ class TaskSeq():
         for chapter in sorted(chapters):
             if chapter!='General':
                 msg+=helpchapter(chapters,chapter)
-            
+
         return msg
 
 
@@ -398,43 +410,63 @@ class TaskSeq():
     def runcmd(self, cmd, data, args):
         return self.tp[cmd].run(self,data,args)
 
-       
+
+    def should_trace(self):
+
+        if not self.trace:
+            return False
+
+        if not self.dumpname:
+            return True # trace all
+
+        if self.iname == self.dumpname:
+            return True
+
+        return False
    #
    # TaskSeq.run
    #
-   
+
     def run(self,data=None,steps=None):
 
         def strcmd(r):
             arg=''
             for k,v in r.items():
                 if k!='command':
-                    arg+="{}={} ".format(k,v)    
+                    arg+="{}={} ".format(k,v)
             return "{} {}".format(r['command'],arg)
 
 
-        for r in self.fullroute:        
+        for r in self.fullroute:
 
-            if self.steps is not None:                
-                if self.steps == 0:                
+            if self.steps is not None:
+                if self.steps == 0:
                     if self.dumpname is None or self.dumpname == self.iname:
                         for d in self.dump:
                             self.runcmd(d,data,dict())
                         self.stop()
+                        data = None
                         break
-                else:                     
+                else:
                     self.steps -= 1
-                                  
+
             self.lastcmd = self.route.pop(0)
-    
+
             r = r.copy()
 
             cmd = r['command']
-                                
-            self.oc.log.debug('RUN {}'.format(strcmd(r)))
-            
+
+            if self.should_trace():
+                self.oc.log.info(u'TRACE RUN {}'.format(strcmd(r)))
+                # dump variables
+                self.oc.log.info("variables:")
+                for vn,vv in self.v.items():
+                    self.oc.log.info("    {} = {}".format(vn, str(vv)[:50]))
+
+            self.oc.log.debug(u'RUN {}'.format(strcmd(r)))
+
             vv = self.getvars()
-            
+
             if isinstance(data,six.string_types):
                 vv['_str'] = data
 
@@ -443,7 +475,7 @@ class TaskSeq():
                     varname = "_dict_{}".format(k)
                     vv[varname]=v
 
-                           
+
             # replace vars
             for argname,argval in r.items():
                 #print("replace arg {} = {}".format(argname, argval))
@@ -452,57 +484,61 @@ class TaskSeq():
                 #print("newval: {}".format(newval))
                 r[argname] = newval
 
-            
+
             if not cmd in self.tp:
-                raise OkerrBadMethod('no method {}'.format(cmd))                   
-                        
+                raise OkerrBadMethod(u'no method {}'.format(cmd))
+
             # RUN task processor
             #print("run, args: ",str(r))
             #data = self.tp[cmd].run(self,data,r)
             data = self.runcmd(cmd,data,r)
-            
+
+            if self.should_trace():
+                self.oc.log.info(u'TRACE DATA {}'.format(json.dumps(data)))
+
+
             #self.oc.log.debug('output: {}'.format(data))
 
             if self._stop:
-                self.oc.log.debug('sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
+                self.oc.log.debug(u'sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
                 data = self.except_data
                 break
 
         # END OF ROUTE
-    
+
         # print('i: {}, data: {}'.format(self.iname,data))
-        
-        if self.iname is not None: 
+
+        if self.iname is not None:
             if data is not None:
                 try:
-                    self.oc.update(self.iname,data,details=self.details,
-                        method=self.method,tags=self.tags,error=None, origkeypath = ':'.join(self.origkeypath), 
+                    self.oc.update(self.iname, data, details=self.details,
+                        policy=self.policy,
+                        method=self.method, tags=self.tags, error=None, origkeypath = ':'.join(self.origkeypath),
                         keypath = ':'.join(self.keypath))
                 except OkerrExc as e:
-                    self.oc.log.error("update failed: {}".format(str(e)))
+                    self.oc.log.error(u"update failed: {}".format(e))
             else:
-                self.oc.log.debug('do no update indicator {}, because data is None'.format(self.iname))
+                self.oc.log.debug(u'do no update indicator {}, because data is None'.format(self.iname))
         else:
-
-            self.oc.log.debug('do not update indicator, because no indicator name')
+            self.oc.log.debug(u'do not update indicator, because no indicator name')
 
 
 
 class TaskAlias(TaskProcessor):
     chapter='Alias'
-    
+
     alias=[]
-    
-       
+
+
     def fullhelp(self):
         if sys.version_info.major == 2:
             msg = super(TaskAlias, self).fullhelp()
         else:
             msg = super().fullhelp()
-        
+
         msg += '\tAlias for: '+str(self.alias)+'\n'
         return msg
-    
+
     def run(self,ts,data,args):
         for code in self.alias:
             cargs = ts.mkargs(code)
@@ -510,7 +546,7 @@ class TaskAlias(TaskProcessor):
             for k,v in args.items():
                 if k in cargs:
                     cargs[k] = v
-            
+
             # print("ALIAS run {} cargs {}".format(code, cargs))
             data = ts.tp[code].run(ts,data,cargs)
 
@@ -531,7 +567,7 @@ class TaskMyName(GenTaskProcessor):
     help = 'returns self name'
     def run(self,ts,data,args):
         return self.code
-    
+
     @property
     def help(self):
         return 'returns {}'.format(self.code)
@@ -543,7 +579,7 @@ TaskMyName('ERR',TaskSeq)
 class TaskEval(GenTaskProcessor):
     parse_args=False
     store_argline='expr'
-    
+
     defargs= { 'expr': ''}
 
     help = 'Safe evaluate pythonic expression'
@@ -551,19 +587,17 @@ class TaskEval(GenTaskProcessor):
     def run(self,ts,data,args):
         try:
             node = evalidate.evalidate(args['expr'])
-            code = compile(node,'<usercode>','eval')            
+            code = compile(node,'<usercode>','eval')
         except (ValueError, SyntaxError) as e:
-            #print(str(e))
-            # raise OkerrBadMethod('Error in filter expression: {}'.format(e))
-            ts.oc.log.error('Error in filter expression: {}'.format(e))            
+            ts.oc.log.error('Error in filter expression: {}'.format(e))
             ts.stop()
             return None
-        
+
         vv = ts.getvars()
         vv['_data']=data
 
         r = eval(code, {}, vv)
-        
+
         if isinstance(r,bool):
             if r:
                 return "OK"
@@ -580,36 +614,47 @@ class TaskExcept(GenTaskProcessor):
 
     parse_args=False
     store_argline='data'
-    
+
     defargs= { 'data': ''}
 
     def run(self,ts,data,args):
         ts.except_data = args['data']
         return data
-    
+
 TaskExcept('EXCEPT',TaskSeq)
 
 class TaskErrors(GenTaskProcessor):
     help = 'okerrclient errors summary'
 
     def run(self,ts,data,args):
-        if last_error:
-            ts.details = "{} ({}s ago)".format(ts.oc.last_error, dhms(time.time() - ts.oc.last_error_time))        
+        if ts.oc.last_error and time.time() > ts.oc.last_error_time:
+            # clear error
+            ts.oc.errors = 0
+            ts.oc.last_error = ''
+            ts.oc.last_error_time = 0
+
+
+
+        if ts.oc.last_error:
+            ts.details = "#{} {} ago: {}".format(
+                ts.oc.errors,
+                dhms(time.time() - ts.oc.last_error_time),
+                ts.oc.last_error)
         else:
             ts.details = ""
         return ts.oc.errors
-    
+
 TaskErrors('ERRORS',TaskSeq)
 
 
 
 class TaskSet(GenTaskProcessor):
-    
+
     parse_args=False
     store_argline='set'
-    
+
     defargs= { 'set': ''}
-    
+
     help = 'Set variable'
     def run(self,ts,data,args):
         setshlex = shlex.shlex(args['set'])
@@ -620,23 +665,23 @@ class TaskSet(GenTaskProcessor):
             ts.oc.log.error('Cannot parse SET statement: {} . Should be in form SET varname=value or varname="some value"'.format(args['set']))
             return None
         ts.setvar(setstat[0],setstat[1])
-        
+
         return data
-    
-   
+
+
 TaskSet('SET',TaskSeq)
 
 
 class TaskExport(GenTaskProcessor):
-    
+
     parse_args=False
     store_argline='exp'
-    
+
     defargs= { 'exp': '' }
-    
+
     help = 'Export data (string) or string element of dict (string) to sequence variable'
     def run(self,ts,data,args):
-    
+
         kv = re.split('\s*=\s*',args['exp'])
         if len(kv)==1 and isinstance(data,six.string_types):
             # this is string
@@ -645,24 +690,24 @@ class TaskExport(GenTaskProcessor):
             ts.setvar(kv[0],data[kv[1]])
         else:
             ts.oc.log.error('export error. Exp: {}, Data: {}'.format(args['exp'],str(data)[:50]))
-                                        
+
         return data
-    
+
 TaskExport('EXPORT',TaskSeq)
 
 
 class TaskTag(GenTaskProcessor):
-    
+
     parse_args=False
     store_argline='tag'
-    
+
     defargs= { 'tag': ''}
-    
+
     help = 'Set tag'
     def run(self,ts,data,args):
-        ts.settag(args['tag'])        
+        ts.settag(args['tag'])
         return data
-    
+
 TaskTag('TAG',TaskSeq)
 
 
@@ -677,16 +722,16 @@ class TaskVersion(GenTaskProcessor):
             ts.details = d['details']
             return d['status']
         except Exception as e:
-            ts.details = "version check error: {}".format(str(e))
-            ts.oc.error("version check error: {}".format(str(e)))
+            ts.details = u"version check error: {}".format(e)
+            ts.oc.error(u"version check error: {}".format(e))
             return "ERR"
-        
+
 TaskVersion('VERSION', TaskSeq)
 
 
 class TaskStr(GenTaskProcessor):
     help = 'if str argument is set, returns str. Joins list by \n, otherwise returns str(data) or empty string.'
- 
+
     store_argline='str'
     parse_args=False
 
@@ -697,12 +742,12 @@ class TaskStr(GenTaskProcessor):
         else:
             if data is None:
                 return ''
-                        
-            try:       
+
+            try:
                 s = "\n".join(data)
             except:
                 s = str(data)
-    
+
         # now, trim it
         # s = s.strip('\r\n\t ')
         return s
@@ -711,7 +756,7 @@ TaskStr('STR',TaskSeq)
 
 class TaskMethod(GenTaskProcessor):
     help = 'set method for this sequence, e.g. "numerical maxlim=80"'
- 
+
     store_argline='method'
     parse_args=False
 
@@ -725,10 +770,24 @@ class TaskMethod(GenTaskProcessor):
 
 TaskMethod('METHOD',TaskSeq)
 
+class TaskPolicy(GenTaskProcessor):
+    help = 'set policy for this sequence'
+
+    store_argline='policy'
+    parse_args=False
+
+    defargs = {'policy': 'Default'}
+    def run(self,ts,data,args):
+        ts.policy=args['policy']
+        return data
+
+TaskPolicy('POLICY', TaskSeq)
+
+
 
 class TaskName(GenTaskProcessor):
     help = 'set new name for this sequence, e.g. "$_name:testname"'
- 
+
     store_argline='name'
     parse_args=False
 
@@ -736,12 +795,12 @@ class TaskName(GenTaskProcessor):
     def run(self,ts,data,args):
         vv = ts.getvars()
         if isinstance(data,dict):
-            vv.update(data) 
-               
-        ts.oc.log.debug(u"NAME: {}".format(args['name']))                
+            vv.update(data)
+
+        ts.oc.log.debug(u"NAME: {}".format(args['name']))
         ts.iname = string.Template(args['name']).safe_substitute(vv)
         #ts.iname = args['name']
-        
+
         return data
 
 TaskName('NAME',TaskSeq)
@@ -760,7 +819,7 @@ class TaskDetails(GenTaskProcessor):
 
     def run(self,ts,data,args):
         outstr=''
-                
+
         fmt=args['format']
 
         if data is None:
@@ -768,19 +827,19 @@ class TaskDetails(GenTaskProcessor):
             return None # can return data. anyway it's None
 
         if fmt:
-            if isinstance(data,dict):       
-                etpl = string.Template(fmt)            
-                ts.details = etpl.safe_substitute(data)            
+            if isinstance(data,dict):
+                etpl = string.Template(fmt)
+                ts.details = etpl.safe_substitute(data)
             else:
-                ts.details = fmt            
-             
+                ts.details = fmt
+
         else:
             ts.details = str(data)
-        
 
-        # return same data 
+
+        # return same data
         return data
-                
+
 TaskDetails('DETAILS',TaskSeq)
 
 
@@ -803,10 +862,10 @@ class DefaultProc(GenTaskProcessor):
                 ts.oc.log.error("{}: not valid default JSON: {}".format(self.code, args['def']))
                 ts.stop()
                 return None
-                
+
         return data
-                
-DefaultProc('DEFAULT', TaskSeq)        
+
+DefaultProc('DEFAULT', TaskSeq)
 
 
 class TaskInt(GenTaskProcessor):
@@ -814,7 +873,7 @@ class TaskInt(GenTaskProcessor):
     def run(self,ts,data,args):
         if isinstance(data,str) or isinstance(data,float) or isinstance(data,int):
             return int(float(data))
-        else:            
+        else:
             return None
 
 TaskInt('INT',TaskSeq)
@@ -822,7 +881,7 @@ TaskInt('INT',TaskSeq)
 
 class TaskNonZero(GenTaskProcessor):
     help = 'Returns OK if input is number higher then 0, or returns ERR otherwise'
-    def run(self,ts,data,args):        
+    def run(self,ts,data,args):
         try:
             if int(float(data))>0:
                 return 'OK'
@@ -840,39 +899,39 @@ class TaskOkErrNot(GenTaskProcessor):
         if data == 'ERR':
             return 'OK'
         return 'ERR'
-        
+
 TaskOkErrNot('OKERRNOT',TaskSeq)
 
 class TaskNone(GenTaskProcessor):
     help = 'returns None. (indicator will not be updated)'
     def run(self,ts,data,args):
         return None
-        
+
 TaskNone('NONE',TaskSeq)
 
 class TaskSort(GenTaskProcessor):
     help = 'sort words or lines in string, or lists or lists of dicts'
     defargs = {'num': '0', 'sep': ' ', 'empty': '0','field': ''}
-        
+
     def run(self,ts,data,args):
         was_str=False
         sep = args['sep'].replace('\\n','\n')
 
         if data is None:
             raise OkerrBadData('attempt to SORT None data')
-    
+
         # make sure we have list
         if isinstance(data,six.string_types):
             was_str=True
             data = data.split(sep)
-        
+
         # delete(?) empty elements
         if not self.argbool(args,'empty'):
             data = list(filter(len,data))
-            
+
         if self.argbool(args,'num'):
             try:
-                data = sorted(data, key=float)        
+                data = sorted(data, key=float)
             except ValueError:
                 pass
         else:
@@ -883,17 +942,17 @@ class TaskSort(GenTaskProcessor):
 
         if was_str:
             data = ' '.join(data)
-        
+
         return data
-        
+
 TaskSort('SORT',TaskSeq)
 
 class TaskDump(GenTaskProcessor):
     help = 'pass data as-is, and print to stdout'
     def run(self,ts,data,args):
-        print(data)        
+        print(data)
         return data
-        
+
 TaskDump('DUMP',TaskSeq)
 
 
@@ -901,7 +960,7 @@ class TaskSeqDump(GenTaskProcessor):
     help = 'pass data as-is, print sequence data to stdout'
     def run(self,ts,data,args):
         datadumplen=100
-    
+
         print("Sequence name '{}': #{}".format(ts.iname,id(ts)), end=" ")
         if ts.parent is None:
             print("no parent")
@@ -911,9 +970,9 @@ class TaskSeqDump(GenTaskProcessor):
         print("route:")
         for r in ts.route:
             print ("\t{} ".format(TaskProcessor.cmd2str(r)))
-            
+
         print("details:",ts.details)
-        print("except:",ts.except_data)        
+        print("except:",ts.except_data)
         print("method:",ts.method)
         print("tags:",ts.tags)
         print("variables:",json.dumps(ts.v, sort_keys=True, indent=4, separators=(',',': ')))
@@ -926,16 +985,16 @@ class TaskSeqDump(GenTaskProcessor):
             else:
                 print ("data: ({}): {}".format(len(datastr),datastr))
         print("\n")
-            
+
         return data
-        
+
 TaskSeqDump('SEQDUMP',TaskSeq)
 
-class TaskJSON(JSONTaskProcessor):              
+class TaskJSON(JSONTaskProcessor):
     help = 'convert any data to JSON string'
     def run(self,ts,data,args):
         return json.dumps(data)
-        
+
 TaskJSON('JSON',TaskSeq)
 
 
@@ -949,8 +1008,8 @@ class TaskFromJSON(JSONTaskProcessor):
             msg = '{}: Try to decode non-JSON data'.format(self.code, data)
             raise OkerrBadData(msg)
             return None
-        return data                
-        
+        return data
+
 TaskFromJSON('FROMJSON',TaskSeq)
 
 
@@ -960,7 +1019,7 @@ class TaskJSONDump(JSONTaskProcessor):
     def run(self,ts,data,args):
         print(json.dumps(data, sort_keys=True, indent=4, separators=(',', ': ')))
         return data
-        
+
 TaskJSONDump('JDUMP',TaskSeq)
 
 class TaskMkSeq(GenTaskProcessor):
@@ -974,83 +1033,85 @@ class TaskMkSeq(GenTaskProcessor):
     defargs = {
         'path': ''
     }
-        
+
     def mkroute(self,ts,data):
         route = list()
-        
+
         for k in sorted(data, key=int):
             e = data[k]
-            #etpl = string.Template(e)            
-            #e = etpl.safe_substitute(v)            
-                        
+            #etpl = string.Template(e)
+            #e = etpl.safe_substitute(v)
+
             args = ts.mkargs(e)
-                       
+
             # cmd = args['command']
-                                    
+
 #            for defarg in ts.tp[cmd].defargs:
 #                if not defarg in e:
 #                    e[defarg]= ts.tp[cmd].defargs[defarg]
-                   
+
             route.append(args)
-        return route 
-    
+        return route
+
     # mkseq.launch
     def launch(self, ts, data,v=None,keypath=None):
-        
+
 
         #name = string.Template(data['name']).safe_substitute(v)
-        
+
         # name = data['name'].format(iname=ts.iname)
 
+
+        # reset variables?
+
         # build route from data
-        route = self.mkroute(ts,data)        
-        
+        route = self.mkroute(ts,data)
+
         ts.oc.log.debug("launch sequence: {}".format(str(route)))
-        
+
         # add remaining route
         route.extend(ts.route)
 
         # print("route:",route)
 
         # newts = okerrclient.taskseq.TaskSeq(ts.iname, None, ts.method)
-        newts = ts.fork() 
-        
+        newts = ts.fork()
+        newts.reset_vars()
         newts.setseq(route)
         if not self.origkeypath is None:
             newts.origkeypath = self.origkeypath
             newts.keypath = keypath
-                    
-                    
+
         newts.run()
 
 
     # mkseq.run
-    def run(self,ts,data,args,v=None,keypath=None):                   
-            
+    def run(self,ts,data,args,v=None,keypath=None):
+
         if keypath is None:
             keypath = list()
-    
-    
+
+
         def hasdict(d):
             for sdname,subdata in d.items():
                 if isinstance(subdata,dict):
                     return True
             return False
-            
-    
+
+
         if data is None :
             # print("no data")
             if args['path']:
                 try:
                     data,path = ts.oc.altkeypath(args['path'])
-                    
+
                     # everything is fine!
-                    self.origkeypath = path.split(':')                
+                    self.origkeypath = path.split(':')
                     self.keypath=list()
                 except OkerrExc as e:
-                    ts.oc.error('get keys error: {}'.format(str(e)))
-                    return None                
-                    
+                    ts.oc.error(u'get keys error: {}'.format(e))
+                    return None
+
             else:
                 ts.oc.log.debug('will not MKSEQ because data is None and no path given (or empty)')
                 return None
@@ -1058,10 +1119,10 @@ class TaskMkSeq(GenTaskProcessor):
             # print("have data")
             pass
 
-    
+
         if v is None:
-            v=dict()   
-                        
+            v=dict()
+
         if isinstance(data,list):
             ddata={}
             # convert data to dict
@@ -1070,29 +1131,28 @@ class TaskMkSeq(GenTaskProcessor):
                 if ss:
                     ddata[i] = ss
             data = ddata
-     
-        # clean up this dict from commends
+
+        # clean up this dict from comments
         data = {k: v for k, v in data.iteritems() if isinstance(v, dict) or not v.startswith('#')}
-                        
+
         if isinstance(data,dict) and hasdict(data):
             for sdname,subdata in data.items():
                 if isinstance(subdata,str):
-                    v[sdname]=subdata    
-                    
-                                
+                    v[sdname]=subdata
+
             for sdname,subdata in data.items():
                 if isinstance(subdata,dict):
                     newkp = keypath[:]
                     newkp.append(sdname)
-                    self.run(ts, subdata, args, v, newkp)                  
+                    self.run(ts, subdata, args, v, newkp)
         else:
             self.launch(ts,data,v,keypath)
-        
+
         # stop this sequence
         ts.stop()
         return None
-    
-    
+
+
 TaskMkSeq('MKSEQ', TaskSeq)
 
 
@@ -1104,16 +1164,16 @@ class TaskDateTime(GenTaskProcessor):
         'offset': '0'
     }
 
-    def run(self,ts,data,args):               
+    def run(self,ts,data,args):
         moment = time.time()
         moment += int(args['offset'])
         m = datetime.datetime.fromtimestamp(moment)
-        
-        
+
+
         prefix = args['prefix']
         ts.setvar(prefix+'day', m.day)
         ts.setvar(prefix+'month', m.month)
-        ts.setvar(prefix+'year', m.year) 
+        ts.setvar(prefix+'year', m.year)
 
         ts.setvar(prefix+'dd',"%02d" % m.day)
         ts.setvar(prefix+'mm',"%02d" % m.month)
@@ -1121,7 +1181,7 @@ class TaskDateTime(GenTaskProcessor):
 
         ts.setvar(prefix+'hour', m.hour)
         ts.setvar(prefix+'minute', m.minute)
-        ts.setvar(prefix+'second', m.second) 
+        ts.setvar(prefix+'second', m.second)
 
         ts.setvar(prefix+'HH',"%02d" % m.hour)
         ts.setvar(prefix+'MM',"%02d" % m.minute)
@@ -1145,11 +1205,11 @@ class TaskKeyVal(GenTaskProcessor):
         'path': '',
     }
 
-    def run(self,ts,data,args):               
+    def run(self,ts,data,args):
         #print ("get keyval data from path: {}".format(args['path']))
         #print("args:", args)
         data = ts.oc.keypath(args['path'])
-    
+
         if data is None:
             ts.oc.log.error('Failed to get keyval. Maybe you forgot to create path \'{}\' in okerr web interface?'.format(args['path']))
             ts.stop()
@@ -1168,13 +1228,13 @@ class TaskSave(GenTaskProcessor):
         'varname': '',
     }
 
-    def run(self,ts,data,args):               
+    def run(self,ts,data,args):
         if not args['varname'] and isinstance(data, dict):
             for k,v in data.items():
-                ts.setvar(k, v) 
+                ts.setvar(k, v)
         else:
-            ts.setvar(args['varname'], data)    
-        
+            ts.setvar(args['varname'], data)
+
         return data
 
 TaskSave('SAVE', TaskSeq)
@@ -1189,8 +1249,8 @@ class TaskLoad(GenTaskProcessor):
         'varname': '',
     }
 
-    def run(self,ts,data,args):               
-        return ts.getvar(args['varname'])    
+    def run(self,ts,data,args):
+        return ts.getvar(args['varname'])
 
 TaskLoad('LOAD', TaskSeq)
 
@@ -1220,27 +1280,27 @@ class TaskDHMS(FormatTaskProcessor):
                 sec-=n*t[k]
                 out+="%d%s%s" % (n,k,sep)
         return out.strip()
-    
+
 
     def run(self,ts,data,args):
-    
+
         if isinstance(data,list):
             return map(lambda d: self.run(ts,d,args), data)
-            
+
         if isinstance(data,dict):
-            field = args['field']                        
+            field = args['field']
             if field:
                 dst = args['destfield']
                 if not dst:
                     dst = field
                 sec = int(float(data[field]))
                 data[dst] = TaskDHMS.dhms_short(sec, ' ', int(args['num']))
-                return data                         
-                   
-                   
+                return data
+
+
         sec = int(float(data)) # in case if data is string or fraction
         return TaskDHMS.dhms_short(sec,' ', int(args['num']))
-        
+
 
 TaskDHMS('DHMS', TaskSeq)
 
@@ -1264,42 +1324,35 @@ class TaskKMGT(FormatTaskProcessor):
             'G': pow(1024,3),
             'T': pow(1024,4),
             '': 1}
-            
+
         for k in sorted(t,key=t.__getitem__,reverse=True):
             fmul = pow(10,frac)
-        
+
             if sz>=t[k]:
                 #n = int((float(sz)*fmul / t[k]))
-                n = sz/float(t[k])                
+                n = sz/float(t[k])
                 #n = n/float(fmul)
-                
+
                 tpl = "{:."+str(frac)+"f}{}"
-                
+
                 return tpl.format(n,k)
 
-    def run(self,ts,data,args):               
+    def run(self,ts,data,args):
 
         if isinstance(data,list):
             return map(lambda d: self.run(ts,d,args), data)
-            
+
         if isinstance(data,dict):
-            field = args['field']                        
+            field = args['field']
             if field:
                 dst = args['destfield']
                 if not dst:
                     dst = field
                 sz = int(float(data[field]))
                 data[dst] = TaskKMGT.kmgt_short(sz, int(args['frac']))
-                return data                         
+                return data
 
         sz = int(float(data)) # in case if data is string or fra
         return TaskKMGT.kmgt_short(sz,int(args['frac']))
 
 TaskKMGT('KMGT', TaskSeq)
-
-
-
-
-
-
-
